#define IM_ROUND_8(x) ((int)((x)+0.5))
#define IM_ROUND_double(x) (x)
#define IM_LIMIT_8(x) ((x) < 0 ? 0 : (x) > 255 ? 255 : (x))
#define IM_LIMIT_double(x) ((x) < 0.0 ? 0.0 : (x) > 1.0 ? 1.0 : (x))
#line 1 "gaussian.im"
#define IMAGER_NO_CONTEXT
#include "imager.h"
#include <math.h>

static double
gauss(int x, double std) {
  return 1.0/(sqrt(2.0*PI)*std)*exp(-(double)(x)*(double)(x)/(2*std*std));
}

/* Counters are as follows
 l:  lines
 i:  columns
 c:  filter coeffs
 ch: channels
 pc: coeff equalization
*/



int
i_gaussian(i_img *im, double stddev) {
  int i, c, ch;
  i_img_dim x, y;
  double pc;
  double *coeff;
  double res[MAXCHANNELS];
  i_img *timg;
  int radius, diameter;
  dIMCTXim(im);

  im_log((aIMCTX, 1,"i_gaussian(im %p, stdev %.2f)\n",im,stddev));
  i_clear_error();

  if (stddev <= 0) {
    i_push_error(0, "stddev must be positive");
    return 0;
  }
  /* totally silly cutoff */
  if (stddev > 1000) {
    stddev = 1000;
  }
 
  timg = i_sametype(im, im->xsize, im->ysize);

  if (im->bits <= 8)
    radius = ceil(2 * stddev);
  else
    radius = ceil(3 * stddev);

  diameter = 1 + radius * 2;

  coeff = mymalloc(sizeof(double) * diameter);

  for(i=0;i <= radius;i++) 
    coeff[radius + i]=coeff[radius - i]=gauss(i, stddev);
  pc=0;
  for(i=0; i < diameter; i++)
    pc+=coeff[i];
  for(i=0;i < diameter;i++)
    coeff[i] /= pc;


#line 63 "gaussian.im"
  if (im->bits <= 8) {
#undef IM_EIGHT_BIT
#define IM_EIGHT_BIT 1
#undef IM_FILL_COMBINE
#define IM_FILL_COMBINE(fill) ((fill)->combine)
#undef IM_FILL_FILLER
#define IM_FILL_FILLER(fill) ((fill)->f_fill_with_color)
#line 64 "gaussian.im"
  i_color rcolor;
  for(y = 0; y < im->ysize; y++) {
    for(x = 0; x < im->xsize; x++) {
      pc=0.0;
      for(ch=0;ch<im->channels;ch++) 
	res[ch]=0; 
      for(c = 0;c < diameter; c++)
	if (i_gpix(im,x+c-radius,y,&rcolor)!=-1) {
	  for(ch=0;ch<im->channels;ch++) 
	    res[ch]+= rcolor.channel[ch] * coeff[c];
	  pc+=coeff[c];
	}
      for(ch=0;ch<im->channels;ch++) {
	double value = res[ch] / pc;
	rcolor.channel[ch] = value > 255 ? 255 : IM_ROUND_8(value);
      }
      i_ppix(timg, x, y, &rcolor);
    }
  }
  
  for(x = 0;x < im->xsize; x++) {
    for(y = 0; y < im->ysize; y++) {
      pc=0.0;
      for(ch=0; ch<im->channels; ch++)
	res[ch]=0; 
      for(c=0; c < diameter; c++)
	if (i_gpix(timg, x, y+c-radius, &rcolor)!=-1) {
	  for(ch=0;ch<im->channels;ch++) 
	    res[ch]+= rcolor.channel[ch] * coeff[c];
	  pc+=coeff[c];
	}
      for(ch=0;ch<im->channels;ch++) {
	double value = res[ch]/pc;
	rcolor.channel[ch] = value > 255 ? 255 : IM_ROUND_8(value);
      }
      i_ppix(im, x, y, &rcolor);
    }
  }
  }
  else {
#undef IM_EIGHT_BIT
#undef IM_FILL_COMBINE
#define IM_FILL_COMBINE(fill) ((fill)->combinef)
#undef IM_FILL_FILLER
#define IM_FILL_FILLER(fill) ((fill)->f_fill_with_fcolor)
#line 64 "gaussian.im"
  i_fcolor rcolor;
  for(y = 0; y < im->ysize; y++) {
    for(x = 0; x < im->xsize; x++) {
      pc=0.0;
      for(ch=0;ch<im->channels;ch++) 
	res[ch]=0; 
      for(c = 0;c < diameter; c++)
	if (i_gpixf(im,x+c-radius,y,&rcolor)!=-1) {
	  for(ch=0;ch<im->channels;ch++) 
	    res[ch]+= rcolor.channel[ch] * coeff[c];
	  pc+=coeff[c];
	}
      for(ch=0;ch<im->channels;ch++) {
	double value = res[ch] / pc;
	rcolor.channel[ch] = value > 1.0 ? 1.0 : IM_ROUND_double(value);
      }
      i_ppixf(timg, x, y, &rcolor);
    }
  }
  
  for(x = 0;x < im->xsize; x++) {
    for(y = 0; y < im->ysize; y++) {
      pc=0.0;
      for(ch=0; ch<im->channels; ch++)
	res[ch]=0; 
      for(c=0; c < diameter; c++)
	if (i_gpixf(timg, x, y+c-radius, &rcolor)!=-1) {
	  for(ch=0;ch<im->channels;ch++) 
	    res[ch]+= rcolor.channel[ch] * coeff[c];
	  pc+=coeff[c];
	}
      for(ch=0;ch<im->channels;ch++) {
	double value = res[ch]/pc;
	rcolor.channel[ch] = value > 1.0 ? 1.0 : IM_ROUND_double(value);
      }
      i_ppixf(im, x, y, &rcolor);
    }
  }
  }
#line 103 "gaussian.im"
  myfree(coeff);
  i_img_destroy(timg);
  
  return 1;
}







